@@ -3,7 +3,7 @@ package Beekeeper::JSONRPC;
 use strict;
 use warnings;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Beekeeper::JSONRPC::Request;
 use Beekeeper::JSONRPC::Notification;
@@ -45,7 +45,7 @@ Beekeeper::JSONRPC - Representation of JSON-RPC objects
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =head1 DESCRIPTION
 
@@ -53,6 +53,11 @@ All Beekeeper RPC calls follow the JSON-RPC 2.0 specification (see L<http://www.
 
 Constructors on this class are not actually used and are provided just for completeness.
 
+=head1 SEE ALSO
+ 
+L<Beekeeper::JSONRPC::Request>, L<Beekeeper::JSONRPC::Response>, L<Beekeeper::JSONRPC::Notification>, 
+L<Beekeeper::JSONRPC::Error>.
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
