@@ -3,7 +3,7 @@ package Beekeeper::MQTT;
 use strict;
 use warnings;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use AnyEvent;
 use AnyEvent::Handle;
@@ -1353,7 +1353,7 @@ sub publish {
         # As client may be syncronous, wait until entire message is sent.
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1733,7 +1733,7 @@ sub flush_buffer {
         # Kernel write buffer is full, see publish() above
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1784,7 +1784,7 @@ Beekeeper::MQTT - Asynchronous MQTT 5.0 client.
  
 =head1 VERSION
  
-Version 0.05
+Version 0.06
 
 =head1 SYNOPSIS
 
@@ -1821,7 +1821,7 @@ Version 0.05
   
   $mqtt->disconnect;
 
-Most methods allows to send arbitrary properties along with commands.
+Most methods allow to send arbitrary properties as key-value pairs of utf8 strings.
 
 Except for trivial cases, error checking is delegated to the server.
 
@@ -1893,10 +1893,7 @@ Callback which is executed after the server accepted the connection.
 
 =head3 disconnect ( %args )
 
-A client can disconnect from the server at anytime by closing the socket but 
-there is no guarantee that the previously sent packets have been received by
-the server. This method should be called to do a graceful shutdown, where the
-client is assured that all previous packets have been received by the server.
+Does a graceful disconnection from the server.
 
 =over 4
 
