@@ -34,7 +34,8 @@ my $toybroker_config_json = qq<
     },
 ]>;
 
-my $TEST_PORT;
+#TODO: Find an unused port
+our $TEST_PORT = 50000 + int(rand(10000));
 
 sub read_config_file {
     my ($class, $file) = @_;
@@ -42,11 +43,6 @@ sub read_config_file {
     my $data = $file eq "bus.config.json"       ? $bus_config_json       : 
                $file eq "toybroker.config.json" ? $toybroker_config_json : '';
 
-    unless ($TEST_PORT) {
-        #TODO: Find an unused port
-        $TEST_PORT = 50000 + int(rand(10000));
-    }
-
     $data =~ s/%PORT%/$TEST_PORT/;
 
     # Allow comments and end-comma
